package fr.jcreigno.blazeds;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Collections;
import java.util.Enumeration;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import flex.messaging.Destination;
import flex.messaging.config.ConfigMap;
import flex.messaging.log.Log;
import flex.messaging.log.Logger;
import flex.messaging.services.AbstractBootstrapService;
import flex.messaging.services.Service;

/**
 * {@link AbstractBootstrapService} implementation which scans all 'META-INF/remoting-config.xml' in classpath. For each
 * destinations found a Flex remoting destination is created.
 * <p>
 * For usage, simply define the following service in your services-config:
 * 
 * <pre class="code">
 * &lt;services&gt;
 * 
 *     ... other service definitions ...
 * 
 *     &lt;service id=&quot;spring-autowiring-bootstrap-service&quot; 
 *              class=&quot;fr.covea.troisma.flex.AutowiringBootstrapService&quot;/&gt;
 *     
 *     &lt;default-channels&gt;
 *         &lt;channel ref=&quot;my-amf&quot;/&gt;
 *     &lt;/default-channels&gt;     
 *     
 * &lt;/services&gt;
 * </pre>
 * 
 * @author jcreigno
 */
public class AutowiringBootstrapService extends AbstractBootstrapService {

    /**
     * Remoting service id config key.
     */
    private static final String SERVICE_ID_KEY = "serviceId";

    /**
     * Remoting service id.
     */
    private static final String DEFAULT_SERVICE_ID = "remoting-service";

    /**
     * Files containing remoting config.
     */
    private static final String REMOTINGS_KEY = "remotings";

    /**
     * Files containing remoting config.
     */
    private static final String DEFAULT_REMOTINGS = "META-INF/remoting-config.xml";

    /**
     * logger.
     */
    private static Logger logger = null;

    /**
     * Remoting config resource list.
     */
    private List<URL> resources = null;

    /**
     * actual remoting service.
     */
    Service service;

    /**
     * Sax parser factory.
     */
    private SAXParserFactory parserFactory;

    /**
     * Default constructeur.
     */
    public AutowiringBootstrapService() {
        super();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void initialize(String id, ConfigMap properties) {
        logger = Log.getLogger(AutowiringBootstrapService.class.getName());
        String serviceId = properties.getPropertyAsString(SERVICE_ID_KEY, DEFAULT_SERVICE_ID);
        service = broker.getService(serviceId);
        parserFactory = SAXParserFactory.newInstance();
        if (service == null) {
            logger.error("Unable to find remoting service with id " + serviceId);
            return;
        }
        try {
            Enumeration<URL> enumeration = Thread.currentThread().getContextClassLoader().getResources(
                    properties.getPropertyAsString(REMOTINGS_KEY, DEFAULT_REMOTINGS));
            resources = Collections.list(enumeration);
        } catch (IOException e) {
            logger.error("Unable to load additionnals configurations : " + e, e);
            return;
        }
        for (URL resource : resources) {
            List<Destination> destinations = null;
            logger.info("reading : " + resource);
            try {
                InputStream stream = resource.openStream();
                destinations = (List<Destination>) parse(stream);
                stream.close();
                for (Destination destination : destinations) {
                    service.addDestination(destination);
                    logger.debug("registering : " + destination);
                }
            } catch (IOException e) {
                logger.error("Unable to read destinations from : " + resource, e);
            } catch (SAXException e) {
                logger.error("Unable to parse destinations from : " + resource, e);
            }
        }
    }

    /**
     * Parses stream to retrieve destinations.
     * @param stream input stream
     * @return destinations list.
     * @throws SAXException if parse error occurs
     * @throws IOException if IO error occurs.
     */
    private List<Destination> parse(InputStream stream) throws SAXException, IOException {

        XMLReader reader = null;
        try {
            reader = (XMLReader) parserFactory.newSAXParser().getXMLReader();
        } catch (ParserConfigurationException e) {
            throw new SAXException("Unable to create SAX parser " + e, e);
        }
        DefaultHandlerExtension extension = new DefaultHandlerExtension(this);
        reader.setContentHandler(extension);
        reader.parse(new InputSource(stream));
        return extension.result;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void start() {
        logger.info("Starting wiring service.");
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void stop() {
        logger.info("Stopping wiring service.");
    }

}
